from django.middleware import csrf
from django.contrib.sessions import middleware


class SessionMiddleware(middleware.SessionMiddleware):
    def __init__(self, get_response=None):
        super(SessionMiddleware, self).__init__(get_response)
        bases = (SessionHeaderMixin, self.SessionStore)
        self.SessionStore = type('SessionStore', bases, {})

    def process_request(self, request):
        super(SessionMiddleware, self).process_request(request)
        sessionid = request.META.get(u'HTTP_X_SESSIONID')
        if sessionid:
            request.session = self.SessionStore(sessionid)
            request.session.csrf_exempt = True

    def process_response(self, request, response):
        supr = super(SessionMiddleware, self)
        response = supr.process_response(request, response)
        if request.session.session_key:
            response['X-SessionID'] = request.session.session_key
        return response


class CsrfViewMiddleware(csrf.CsrfViewMiddleware):
    def process_view(self, request, *args, **kwargs):
        if not request.session.csrf_exempt:
            supr = super(CsrfViewMiddleware, self)
            return supr.process_view(request, *args, **kwargs)


class SessionHeaderMixin(object):
    def __init__(self, session_key=None):
        super(SessionHeaderMixin, self).__init__(session_key)
        self.csrf_exempt = False



from django.conf import settings

def pytest_configure():
    settings.configure(
        SESSION_ENGINE='django.contrib.sessions.backends.cache',
        CACHES={
            'default': {
                'BACKEND': 'django.core.cache.backends.dummy.DummyCache',
            }
        }
    )



from django.http import HttpResponse
import django_session_header.middleware


class TestSessionMiddleware(object):
    def test_init(self):
        """The SessionStore should create an instance with csrf_exempt."""
        middleware = django_session_header.middleware.SessionMiddleware()
        store = middleware.SessionStore()
        assert not store.csrf_exempt

    def test_process_request_normal(self, rf):
        """Should not be CSRF exempt by default."""
        middleware = django_session_header.middleware.SessionMiddleware()
        request = rf.post("/")
        middleware.process_request(request)
        assert not request.session.csrf_exempt

    def test_process_request_header(self, rf):
        """Should be CSRF exempt with session header."""
        middleware = django_session_header.middleware.SessionMiddleware()
        request = rf.post("/")
        request.META["HTTP_X_SESSIONID"] = "abcdefghijklmnopqrstuvwxyz"
        middleware.process_request(request)
        assert request.session.csrf_exempt

    def test_process_response_session_empty(self, rf):
        """There should be no sessionid header if the session is empty."""
        middleware = django_session_header.middleware.SessionMiddleware()
        request = rf.get("/")
        middleware.process_request(request)
        response = HttpResponse()
        response = middleware.process_response(request, response)
        assert "X-SessionID" not in response

    def test_process_response_session_nonempty(self, rf):
        """There should be a sessionid header if the session is not empty."""
        middleware = django_session_header.middleware.SessionMiddleware()
        request = rf.get("/")
        middleware.process_request(request)
        response = HttpResponse()
        request.session["spam"] = "eggs"
        response = middleware.process_response(request, response)
        assert "X-SessionID" in response


class TestCsrfViewMiddleware(object):
    def test_process_request_normal(self, rf):
        """CSRF should block if no header."""
        session_middleware = django_session_header.middleware.SessionMiddleware()
        csrf_middleware = django_session_header.middleware.CsrfViewMiddleware()
        request = rf.post("/")
        session_middleware.process_request(request)
        view = lambda r: HttpResponse()
        response = csrf_middleware.process_view(request, view, (), {})
        assert response is not None

    def test_process_request_exempt(self, rf):
        """CSRF should not block if header is present."""
        session_middleware = django_session_header.middleware.SessionMiddleware()
        csrf_middleware = django_session_header.middleware.CsrfViewMiddleware()
        request = rf.post("/")
        request.META["HTTP_X_SESSIONID"] = "abcdefghijklmnopqrstuvwxyz"
        session_middleware.process_request(request)
        view = lambda r: HttpResponse()
        response = csrf_middleware.process_view(request, view, (), {})
        assert response is None



from rest_framework.authentication import SessionAuthentication


class SessionAuthentication(SessionAuthentication):
    """
    Look for X-SessionID header usage, and ignore CSRF.
    """

    def enforce_csrf(self, request):
        """
        Don't enforce CSRF if using a session header.
        """
        if not request.session.csrf_exempt:
            return super(SessionAuthentication, self).enforce_csrf(request)



import pytest
from rest_framework.exceptions import PermissionDenied
from rest_framework.request import Request
import django_session_header.authentication


class TestSessionAuthentication(object):
    def test_enforce_csrf_normal(self, rf):
        """Should cause CSRF failure normally."""
        middleware = django_session_header.middleware.SessionMiddleware()
        authentication = django_session_header.authentication.SessionAuthentication()
        request = rf.post("/")
        middleware.process_request(request)
        with pytest.raises(PermissionDenied):
            authentication.enforce_csrf(Request(request))

    def test_enforce_csrf_session_header(self, rf):
        """Should not cause CSRF failure when using a header."""
        middleware = django_session_header.middleware.SessionMiddleware()
        authentication = django_session_header.authentication.SessionAuthentication()
        request = rf.post("/")
        request.META["HTTP_X_SESSIONID"] = "abcdefghijklmnopqrstuvwxyz"
        middleware.process_request(request)
        authentication.enforce_csrf(Request(request))



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['django_session_header_11']

package_data = \
{'': ['*']}

install_requires = \
['django>=1.11,<2.0']

extras_require = \
{'drf': ['djangorestframework>=3.9,<4.0']}

setup_kwargs = {
    'name': 'django-session-header-11',
    'version': '1.0.1',
    'description': 'Identify the Django Session by a Header',
    'long_description': "Django Session Header: Identify the session through a header\n============================================================\n\nThere are some situations where the browser\nmay not allow any cookies at all to be used.\nIn those cases, we would like to be able to fall back\nto something that is both secure, and capable.\nThis package allows you to manually pass the\nsessionid using a header, so that you can continue\nto use Django's excellent session management.\n\nIt extends Django's built-in sessions to support\nsessions in places where cookies are not allowed.\nFor most views, the handling will be seamless.\nThose that need to have sessions that persist despite the\nabsence of cookies, there are a few extra features.\n\n\nUsage\n=====\n\nFirst, install the package.\n\n.. code-block:: sh\n\n    pip install django-session-header\n\nReplace ``django.contrib.sessions.middleware.SessionMiddleware``\nin your ``settings.py`` with the following:\n\n.. code-block:: python\n\n    MIDDLEWARE_CLASSES = [\n       # ...\n       # 'django.contrib.session.middleware.SessionMiddleware',\n       'django_session_header.middleware.SessionMiddleware',\n    ]\n\nAnd replace the Django Rest Framework ``SessionAuthentication``\nclass with ``django_session_header.authentication.SessionAuthentication``:\n\n.. code-block:: python\n\n    REST_FRAMEWORK = {\n        'DEFAUlT_AUTHENTICATION_CLASSES': [\n            # ...\n            # 'rest_framework.authentication.SessionAuthentication',\n            'django_session_header.authentication.SessionAuthentication',\n        ]\n    }\n\nIf a session was obtained via a session header,\nthen ``request.session.csrf_exempt`` will be ``True``.\nYou can use this to conditionally apply CSRF protection.\nOr, if you prefer, you can replace Django's normal CSRF middleware\nwith ``django_session_header.middleware.CsrfViewMiddleware``:\n\n.. code-block:: python\n\n    MIDDLEWARE_CLASSES = [\n        # ...\n        # 'django.middleware.csrf.CsrfViewMiddleware',\n        'django_session_header.middleware.CsrfViewMiddleware',\n    ]\n\nThe ``sessionid`` will be available in the ``X-SessionID`` response header,\nand you can now set the ``X-SessionID`` header on the request manually\nto avoid needing cookies to power your sessions.\n",
    'author': 'Ryan Hiebert',
    'author_email': 'ryan@ryanhiebert.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ryanhiebert/django-session-header',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
